<?php

namespace Drupal\entity_visitors\EventSubscriber;

use Drupal\entity_visitors\Service\EntityVisitorsManager;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * A subscriber to the route when it matches an entity route.
 */
class EntityRouteMatchSubscriber implements EventSubscriberInterface {

  /**
   * The manager takes the current visited entity and handle things for you.
   *
   * @var \Drupal\entity_visitors\Service\EntityVisitorsManager
   */
  private $entityVisitedManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityVisitorsManager $entityVisitedManager) {
    $this->entityVisitedManager = $entityVisitedManager;
  }

  /**
   * Listen to the requests to check the current route.
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::REQUEST => 'updateEntityVisitors',
    ];
  }

  /**
   * Get the current route then manage the entity if visited.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   The GetResponse event.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function updateEntityVisitors(GetResponseEvent $event) {
    $routeName = $event->getRequest()->get('_route');
    // Match the route with an entity view route.
    if (preg_match('/(entity.)*(.canonical)/', $routeName)) {
      // eg, node, user, etc,.
      $visitedEntityType = explode('.', $routeName)[1];
      $visitedEntityId = $event->getRequest()->get($visitedEntityType)->id();

      $this->entityVisitedManager->handleEntity($routeName, $visitedEntityType, $visitedEntityId);
    }
  }

}
